(*----------------------------------------------------------------------*)
(*         Init_Text_Terminal --- Initialize text mode terminals        *)
(*----------------------------------------------------------------------*)

PROCEDURE Init_Text_Terminal;

(*----------------------------------------------------------------------*)
(*                                                                      *)
(*     Procedure:  Init_Text_Terminal                                   *)
(*                                                                      *)
(*     Purpose:    Initializes text mode terminal emulations.           *)
(*                                                                      *)
(*     Calling Sequence:                                                *)
(*                                                                      *)
(*        Init_Text_Terminal;                                           *)
(*                                                                      *)
(*----------------------------------------------------------------------*)

BEGIN (* Init_Text_Terminal *)
                                   (* Ensure we're in text mode *)
   Do_Status_Time := FALSE;

                                   (* Pick up terminal name     *)

   Short_Terminal_Name := Terminal_Name_List[ ORD( Terminal_To_Emulate ) ];

   IF Current_Video_Mode = HiRes_GraphMode THEN
      BEGIN
         TextMode( Text_Mode );
         IF EGA_Installed THEN
            Set_EGA_Text_Mode( Max_Screen_Line );
         Reset_Global_Colors;
      END;

   Window( 1, 1, Max_Screen_Col, Max_Screen_Line );

   Save_Screen( Saved_Screen );
   Draw_Menu_Frame( 10, 10, 55, 15, Menu_Frame_Color, Menu_Title_Color,
                    Menu_Text_Color, '' );

   WRITELN('Beginning ',
           Long_Terminal_Names[ ORD( Terminal_To_Emulate ) ], ' Emulation');
   DELAY( One_Second_Delay );

   Restore_Screen( Saved_Screen );
   Reset_Global_Colors;
                                   (* Load function keys             *)
   IF Auto_Load_FunKeys THEN
      Load_Function_Keys( Short_Terminal_Name + '.FNC' );

                                   (* Scroll screen up if on last line *)

   IF ( WhereY = Max_Screen_Line ) THEN
      BEGIN
         GoToXY( WhereX , PRED( Max_Screen_Line ) );
         Scroll( 1, Max_Screen_Line, 1, Max_Screen_Col, 1,
                 ForeGround_Color, BackGround_Color );
      END;

END   (* Init_Text_Terminal *);
